/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Scenix.h"

/****************************************************************************
Procdures utiles
****************************************************************************/
//#define fregister (Code&0x1F)
//#define opcode (Code&0xFE0)

char * Register(word Reg)
{
  switch (Reg) {
    case 0x00 : return "INDF";
    case 0x01 : return "RTCC";
    case 0x02 : return "PC";
    case 0x03 : return "STATUS";
    case 0x04 : return "FSR";
    case 0x05 : return "RA";
    case 0x06 : return "RB";
    case 0x07 : return "RC";
    default : {
      static char buffer[8];
      sprintf(buffer,"r%0.2X",Reg);
      return buffer;
    }
  }
}

char * Bits(word Reg)
{
  switch (Reg) {
    case 0x03 : return "C";
    case 0x23 : return "DC";
    case 0x43 : return "Z";
    case 0x63 : return "PD";
    case 0x83 : return "TO";
    default : {
      static char buffer[12];
      sprintf(buffer,"%s.%d",Register((byte)(Reg&0x01F)),Reg>>5);
      return buffer;
    }
  }
}

/****************************************************************************
Classe SxDisassembly
****************************************************************************/
void SxDisassembly::ToggleBreak(int Addr)
{
  if ((Addr>=0)&&(Addr<Sx->RomSize)) {
    Sx->BreakPoints[Addr]=(void*)(!((int)Sx->BreakPoints[Addr]));
  }
}

int SxDisassembly::IsBreak(int Addr)
{
  if ((Addr>=0)&&(Addr<Sx->RomSize)) {
    return (int)(Sx->BreakPoints[Addr]);
  } else return 0;
}

char *SxDisassembly::GetLine(int Addr)
{
  return Sx->Disasm(Addr);
}

char *SxDisassembly::GetSourceCode(int Addr)
{
  if (Sx->Lines[Addr]) return Sx->Lines[Addr];
    else return "";
}

char *SxDisassembly::GetOpCode(int Addr)
{
  static char buffer[5];
  snprintf(buffer,5,"%0.4X",Sx->ROM[Addr]);
  return buffer;
}

int SxDisassembly::GetIP()
{
  return Sx->PC;
}

int SxDisassembly::GetAddr()
{
  return Sx->PC;
}


void SxDisassembly::SetIP(int Addr)
{
  if ((Addr>=0)&&(Addr<Sx->RomSize)) Sx->PC=Addr;
}

void SxDisassembly::SetAddr(int Addr)
{
  if ((Addr>=0)&&(Addr<Sx->RomSize)) Sx->PC=Addr;
}

void SxDisassembly::GetMemRange(int *start, int *end)
{
  if (start) *start=0;
  if (end) *end=Sx->RomSize;
}

void SxDisassembly::ToCursor(int Addr)
{
  Sx->ToCursorBreakAddr = Addr;
}

void SxDisassembly::ClearBreakpoints()
{
  for (int i = 0; i < Sx->RomSize; i++)
  {
    Sx->BreakPoints[i] = NULL;
  }
}

char *SxDisassembly::GetRealAddr(int Addr)
{
  static char buffer[5];
  snprintf(buffer,5,"%0.4X",Addr);
  return buffer;
}

void SxDisassembly::ReloadCode()
{
  Sx->ReloadFiles();
}

SxDisassembly::SxDisassembly(int name, attribute *attribs, symbol *parent) : disassembly(name,attribs,parent)
{
  Sx=(Sx1828*)parent;
}

/****************************************************************************
Classe SxSource
****************************************************************************/

void SxSource::ToggleBreak(int Line)
{
  int Addr;
  while (Line < Sx->NumLines && (Addr = Sx->SourceToAddr[Line]) == -1)
    Line++;

  if ((Addr>=0)&&(Addr<Sx->RomSize)) {
    Sx->BreakPoints[Addr]=(void*)(!((int)Sx->BreakPoints[Addr]));
  }
}

int SxSource::IsBreak(int Line)
{
  int Addr;
  while (Line < Sx->NumLines && (Addr = Sx->SourceToAddr[Line]) == -1)
    Line++;

//  int Addr = Sx->SourceToAddr[Line];

  if ((Addr>=0)&&(Addr<Sx->RomSize)) {
    return (int)(Sx->BreakPoints[Addr]);
  } else return 0;
}

char *SxSource::GetLine(int Line)
{
  int Addr = Sx->SourceToAddr[Line];
  if ((Addr>=0)&&(Addr<Sx->RomSize))
    return Sx->Disasm(Addr);
  else return "";
}

char *SxSource::GetSourceCode(int Line)
{
  if (Line >= 0 && Line < Sx->NumLines)
    return Sx->SourceLines[Line];
  return "";
}

char *SxSource::GetOpCode(int Line)
{
  int Addr = Sx->SourceToAddr[Line];
  static char buffer[5];
  if ((Addr>=0)&&(Addr<Sx->RomSize))
  {
    snprintf(buffer,5,"%0.4X",Sx->ROM[Addr]);
    return buffer;
  }
  else
    return "";
}

int SxSource::GetIP()
{
  return Sx->AddrToSource[Sx->PC];
}

int SxSource::GetAddr()
{
  return Sx->PC;
}

void SxSource::SetIP(int Line)
{
  int Addr = Sx->SourceToAddr[Line];
  if ((Addr>=0)&&(Addr<Sx->RomSize)) Sx->PC=Addr;
}

void SxSource::SetAddr(int Addr)
{
  if ((Addr>=0)&&(Addr<Sx->RomSize)) Sx->PC=Addr;
}

void SxSource::GetMemRange(int *start, int *end)
{
  if (start) *start=0;
  if (end) *end=Sx->NumLines;
}

void SxSource::ToCursor(int Line)
{
  Sx->ToCursorBreakAddr = Sx->SourceToAddr[Line];
}

void SxSource::ClearBreakpoints()
{
  for (int i = 0; i < Sx->RomSize; i++)
  {
    Sx->BreakPoints[i] = NULL;
  }
}

char *SxSource::GetRealAddr(int Line)
{
  int Addr = Sx->SourceToAddr[Line]; 
  static char buffer[5];
  if (Addr >= 0 && Addr < Sx->RomSize)
  {
    snprintf(buffer,5,"%0.4X",Addr);
    return buffer;
  }
  else 
  return "";
}

void SxSource::ReloadCode()
{
  Sx->ReloadFiles();
}

SxSource::SxSource(int name, attribute *attribs, symbol *parent) : disassembly(name,attribs,parent)
{
  Sx=(Sx1828*)parent;
}

/****************************************************************************
Classe SX1828type
****************************************************************************/

component* Sx1828type::create(int name, attribute *attribs, symbol *parent)
{
  return new Sx1828(this,name,attribs,parent,SX_28);
}

/****************************************************************************
Classe SX1828
****************************************************************************/

#define fr_INDF   (regfr(0))
#define fr_RTCC   (RAM[01])
#define fr_PC     (RAM[02])
#define fr_STATUS (RAM[03])
#define fr_FSR    (RAM[04])
#define fr_RA     (RAM[05])
#define fr_RB     (RAM[06])
#define fr_RC     (RAM[07])
#define STATUS_C  1
#define STATUS_DC (1<<1)
#define STATUS_Z  (1<<2)
#define STATUS_PD (1<<3)
#define STATUS_TO (1<<4)

#define GET_C   (fr_STATUS&1)
#define GET_DC  ((fr_STATUS&2)>>1)
#define GET_Z   ((fr_STATUS&4)>>2)
#define GET_PD  ((fr_STATUS&8)>>3)
#define GET_TO  ((fr_STATUS&16)>>4)

#define CHANGE_C(b)  fr_STATUS=(byte)((fr_STATUS&0xFE)|((b)&1))
#define CHANGE_DC(b) fr_STATUS=(byte)((fr_STATUS&0xFD)|(((b)&1)<<1))
#define CHANGE_Z(b)  fr_STATUS=(byte)((fr_STATUS&0xFB)|(((b)&1)<<2))
#define CHANGE_PD(b) fr_STATUS=(byte)((fr_STATUS&0xF7)|(((b)&1)<<3))
#define CHANGE_TO(b) fr_STATUS=(byte)((fr_STATUS&0xEF)|(((b)&1)<<4))

#define CONFIG_CF (Fusex&0x40)
#define TURBO (!(Fuse&0x800))
#define IRC (!(Fuse&0x080))
//#define OPTION_X ((Fuse&0x200)||(Fusex&0x080))
#define OPTION_X (Fusex&0x080)
#define STACK_X  (Fusex&0x080)
#define WDTE     (Fuse&0x004)

#define RTW    (OPTION&0x80)
#define RTE_IE (OPTION&0x40)
#define RTS    (OPTION&0x20)
#define RTE_ES (OPTION&0x10)
#define PSA    (OPTION&0x08)
#define PS012  (OPTION&0x07)

#define CMP_EN  0x80
#define CMP_OE  0x40
#define CMP_RES 0x01


Sx1828::Sx1828(componenttype *type, int name, attribute *attribs, symbol *parent,int SxType)
 : component(type, name, attribs,parent),
OSC1(symbtable.toid("OSC1"),this,attribs,this,(char*)&SpecialPins,0x01),
MCLR(symbtable.toid("MCLR"),this,attribs,this,(char*)&SpecialPins,0x04),
RTCC(symbtable.toid("RTCCPIN"),this,attribs,this,(char*)&SpecialPins,0x02),
OSC2(symbtable.toid("OSC2"),this,attribs,this,(char*)&SpecialPins,0x08)
{
  char buffer[20];
  switch (SxType) {
    case SX_18:
    case SX_20:
    case SX_28:{
      RomSize=2048;
      RamSize=256;   //Only one half of this memory plane is used but it is faster
      break;
    }
  }
  ROM=new word[0x1012]; // This size is necessary to load Fuse and Fusex
  RAM=new byte[RamSize];
  Lines=new (char*[RomSize]);
  BreakPoints=new (void*[RomSize]);  //void* for future use
  SourceLines = (char **) NULL;  
  SourceToAddr = (int *) NULL;
  AddrToSource = (int *) NULL;
  //Allocate pins
  for (int j=0;j<3;j++)
  for (int i=0;(i<8)&&((i<4)||(j>0));i++) {
    sprintf(buffer,"R%c%d",0x41+j,i);
    PORT[j][i]=new pin_3state(symbtable.toid((char*)&buffer),this,attribs,this);
    DefineBit(PORT[j][i]->Tris ,(char*)&TRIS[j],(byte)(1<<i));
    DefineBit(PORT[j][i]->Latch,(char*)&R_OUT[j],(byte)(1<<i));
    DefineBit(PORT[j][i]->Pin  ,(char*)&RAM[5+j],(byte)(1<<i));
    PORT[j][i]->SetTTL(1);
    DefineBit(PORT[j][i]->PullUp,(char*)&PLP[j],(byte)(1<<i));
  }
  ToCursorBreakAddr = -1;
  CycleCount=0;
  updatetime=0;
  UpdateTime1=UpdateTime2=0;
  SystemClock=0;
  Prescaler=0;
  PowerDown=false;

  ReloadFiles();
  Reset(RST_Powerup);
  LastStepID=StepStackPtr=-1;
}

void Sx1828::ReloadFiles()
{
  char *filename;
  //AJOUTER ICI LIBERATION DES ANCIENNES RESSOURCES
  SourceLines = (char **) NULL;
  SourceToAddr = (int *) NULL;
  AddrToSource = (int *) NULL;
  for (int i = 0; i < RomSize; i++) {
    ROM[i] = -1;
    Lines[i] = NULL;
    BreakPoints[i] = NULL;
  }
  attribute *attribs=getattributes();
  if (attribs->getstringattribute(symbtable.toid("file"), &filename)) {
    int len = strlen(filename);
    if (strlen(filename)>3) {
      if (toupper(filename[len-3]) == 'H' && toupper(filename[len-2]) == 'E' && toupper(filename[len-1]) == 'X') {
        if (LoadHexFile(filename)) error(TYPE_ERROR,GetParseLineNum(), GetParseFileName(),"Can't open hex file '%s'"
        ,filename);
      } else
      if (toupper(filename[len-3]) == 'L' && toupper(filename[len-2]) == 'S' && toupper(filename[len-1]) == 'T') {
        if (LoadLstFile(filename)) error(TYPE_ERROR,GetParseLineNum(), GetParseFileName(),"Can't open lst file '%s'"
        ,filename);
      } else error(TYPE_ERROR,GetParseLineNum(), GetParseFileName(),"Bad file type '%s'"
      ,filename);
    } else error(TYPE_ERROR,GetParseLineNum(), GetParseFileName(),"Bad file type '%s'"
      ,filename);
  }

  for (int i = 0; i < RomSize; i++)
    if (ROM[i] == -1)
    {
      ROM[i] = 0xFFF;
      BreakPoints[i] = (void *) 1;
    }

  int val;
  if (attribs->getintattribute("Fuse", &val)) ROM[0x1010]=val&0XFFF;
  if (attribs->getintattribute("Fusex", &val)) ROM[0x1011]=val&0xFFF;
}

/*pin *Sx1828::GetPin(symbolname *name)
{
  if (name == NULL || name->next != NULL) return (pin *) NULL;
  for (int j=0;j<3;j++)
  for (int i=0;(i<8)&&((i<4)||(j>0));i++) {
    if (*(name->elem) == PORT[j][i]->getname()) return PORT[j][i];
  }
  if (*(name->elem) == OSC1.getname()) return &OSC1;
  if (*(name->elem) == OSC2.getname()) return &OSC2;
  if (*(name->elem) == RTCC.getname()) return &RTCC;
  if (*(name->elem) == MCLR.getname()) return &MCLR;
  return (pin *) NULL;
}
*/
list<symbol> *Sx1828::getapparentsymbols()
{
  int i;
  list<int> n(&i);
  char buffer[20];
  i=symbtable.toid("STACK");getsymbol(&n);
  i=symbtable.toid("ROM");getsymbol(&n);
  i=symbtable.toid("BANK_0");getsymbol(&n);
  for (int j=1;j<16;j+=2) {
    sprintf(buffer,"BANK_%X",j);
    i=symbtable.toid(buffer);getsymbol(&n);
  }
  i=symbtable.toid("Code");getsymbol(&n);
  if (AddrToSource) {
    i=symbtable.toid("Source");getsymbol(&n);
  }  
  i=symbtable.toid("PC");getsymbol(&n);
  i=symbtable.toid("FUSE");getsymbol(&n);
  i=symbtable.toid("FUSEX");getsymbol(&n);
  i=symbtable.toid("C");getsymbol(&n);
  i=symbtable.toid("DC");getsymbol(&n);
  i=symbtable.toid("Z");getsymbol(&n);
  i=symbtable.toid("PD");getsymbol(&n);
  i=symbtable.toid("TO");getsymbol(&n);
  i=symbtable.toid("PAGE");getsymbol(&n);
  i=symbtable.toid("OPTION");getsymbol(&n);
  i=symbtable.toid("MODE");getsymbol(&n);
  i=symbtable.toid("W");getsymbol(&n);
  i=symbtable.toid("INDF");getsymbol(&n);
  i=symbtable.toid("RTCC");getsymbol(&n);
  i=symbtable.toid("STATUS");getsymbol(&n);
  i=symbtable.toid("FSR");getsymbol(&n);
  i=symbtable.toid("WKPND_B");getsymbol(&n);
  i=symbtable.toid("WKED_B");getsymbol(&n);
  i=symbtable.toid("WKEN_B");getsymbol(&n);
  for (int j=0;j<3;j++) {
    sprintf(buffer,"R%c",0x41+j);
    i=symbtable.toid(buffer);getsymbol(&n);
    sprintf(buffer,"TRIS%c",0x41+j);
    i=symbtable.toid(buffer);getsymbol(&n);
    sprintf(buffer,"R_OUT%c",0x41+j);
    i=symbtable.toid(buffer);getsymbol(&n);
    sprintf(buffer,"PLP%c",0x41+j);
    i=symbtable.toid(buffer);getsymbol(&n);
  }
  return children;
}

symbol* Sx1828::_getlocalsymbol(symbolname *name)
{
  int id;
  char buffer[20];
  //RAM et ROM
  if (*(name->elem) == (id=symbtable.toid("STACK")) )
    return (new array(id,
      intattribute(symbtable.toid("columns"), 1,
      intattribute(symbtable.toid("datawidth"), 16,
      intattribute(symbtable.toid("base"), 16,
      getattributes())))
      , this, (char*)&Stack[0], 0, 2*StackDepth))->getlocalsymbol(name->next);
#ifndef __linux__
  if (*(name->elem) == (id=symbtable.toid("ROM")) )
    return (new array(id,
      intattribute(symbtable.toid("columns"), 8,
      intattribute(symbtable.toid("datawidth"), 16,
      intattribute(symbtable.toid("base"), 16,
      getattributes())))
    , this, (char*)&ROM[0], 0, RomSize*2))->getlocalsymbol(name->next);
#endif // the GTK arraywidget isn't up to it yet.
  if (*(name->elem) == (id=symbtable.toid("BANK_0")) )
    return (new array(id, getattributes(), this, (char*)RAM, 8, 16))->getlocalsymbol(name->next);
  for (int i=1;i<16;i+=2) {
    sprintf(buffer,"BANK_%X",i);
    if (*(name->elem) == (id=symbtable.toid(buffer)) )
      return (new array(id,
        intattribute(symbtable.toid("columns"), 16,
        intattribute(symbtable.toid("datawidth"), 8,
        intattribute(symbtable.toid("base"), 16,
        intattribute(symbtable.toid("reduced"), 1,
        #ifndef __linux__
        intattribute(symbtable.toid("width"), 450,
        #endif
        getattributes()))))
        #ifndef __linux__
        )
        #endif
      , this, (char*)RAM, i*16, (i+1)*16))->getlocalsymbol(name->next);
  }
  //
  if (*(name->elem) == (id=symbtable.toid("Code")) )
    return (new SxDisassembly(id,getattributes(),this))->getlocalsymbol(name->next);

  if ((AddrToSource)&&(*(name->elem) == (id=symbtable.toid("Source")) ))
    return (new SxSource(id,getattributes(),this))->getlocalsymbol(name->next);

  if ( (*(name->elem) == (symbtable.toid("PC")) )||(*(name->elem) == (symbtable.toid("IP")))) {
    return (new basevariableextract<short int>(symbtable.toid("PC"),
      intattribute(symbtable.toid("base"), 16,
      getattributes())
      , this, &PC, 0, 11 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("FUSE")) ) {
    variable* var=(new basevariableextract<short int>(id,
    intattribute(symbtable.toid("base"), 16,
    getattributes())
    , this, &Fuse, 0, 11 ));
    var->readonly=1;
    return var->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("FUSEX")) ) {
    variable* var=(new basevariableextract<short int>(id,
    intattribute(symbtable.toid("base"), 16,
    getattributes())
    , this, &Fusex, 0, 11 ));
    var->readonly=1;
    return var->getlocalsymbol(name->next);
  }

  //flags
  if (*(name->elem) == (id=symbtable.toid("C")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 0, 0 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("DC")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 1, 1 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("Z")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 2, 2 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("PD")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 3, 3 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("TO")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 4, 4 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("PAGE")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 5, 7))->getlocalsymbol(name->next);
  }
  //Special registers
  if (*(name->elem) == (id=symbtable.toid("W")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &W, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("INDF")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &RAM[0], 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("RTCC")) ) {
    return (new basevariableextract<word>(id, getattributes(), this, &RTCCCounter, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("OPTION")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &OPTION, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("MODE")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &MODE, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("STATUS")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_STATUS, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("FSR")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &fr_FSR, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("WKPND_B")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &WKPND_B, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("WKED_B")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &WKED_B, 0, 7 ))->getlocalsymbol(name->next);
  }
  if (*(name->elem) == (id=symbtable.toid("WKEN_B")) ) {
    return (new basevariableextract<byte>(id, getattributes(), this, &WKED_B, 0, 7 ))->getlocalsymbol(name->next);
  }
  for (int j=0;j<3;j++) {
    sprintf(buffer,"R%c",0x41+j);
    if (*(name->elem) == (id=symbtable.toid(buffer)) ) {
      return (new basevariableextract<byte>(id, getattributes(), this, &RAM[5+j], 0, 7 ))->getlocalsymbol(name->next);
    }
    sprintf(buffer,"TRIS%c",0x41+j);
    if (*(name->elem) == (id=symbtable.toid(buffer)) ) {
      return (new basevariableextract<byte>(id, getattributes(), this, &TRIS[j], 0, 7 ))->getlocalsymbol(name->next);
    }
    sprintf(buffer,"R_OUT%c",0x41+j);
    if (*(name->elem) == (id=symbtable.toid(buffer)) ) {
      return (new basevariableextract<byte>(id, getattributes(), this, &R_OUT[j], 0, 7 ))->getlocalsymbol(name->next);
    }
    sprintf(buffer,"PLP%c",0x41+j);
    if (*(name->elem) == (id=symbtable.toid(buffer)) ) {
      return (new basevariableextract<byte>(id, getattributes(), this, &PLP[j], 0, 7 ))->getlocalsymbol(name->next);
    }
  }
  return (symbol*)NULL;
}

void Sx1828::Update(int force)
{
  real time=Project.GetTime();

  if (SpecialPins&0x04) {  //Test if reset is active
    //Analog Comparator
    if (!(CMP_B&CMP_EN)) {
      //Analog comparator is enabled: compare RB2 and RB1
      if (PORT[1][2]->GetVin()>PORT[1][1]->GetVin()) CMP_B|=CMP_RES;
        else CMP_B&=~CMP_RES;
      // if the output is enabled, put the result in RB0
      if (CMP_B&CMP_OE) {
        R_OUT[1]=(R_OUT[1]&0xFE)|(CMP_B&CMP_RES);
      }
    }
    //handle MIWU pins
    WKPND_B=WKPND_B|(((RAM[6]^LastPortB)&RAM[6]&(~WKED_B))  //rising edge
    |((RAM[6]^LastPortB)&LastPortB&(WKED_B))); //falling edge
    if (WKPND_B&(~WKEN_B)) {
      //Raise an interrupt
      if (PowerDown) Reset(RST_Wakeup);
      else if (!IntStack.IntActive){
        //1:save registers
        IntStack.W=W;
        IntStack.FSR=fr_FSR;
        IntStack.STATUS=fr_STATUS;
        IntStack.PC=PC;
	    IntStack.IntActive = 1;
	    fr_STATUS &= 0x1F; // Clear PAGE
	// #### Set page to zero, check not already in int, only update WKPND!
        //2:branchement
        CycleCount=3;
        PC=0;
        //Test BreakPoint
        TestBreakPoints();
      }
    }
    LastPortB=RAM[6];
    //Watch Dog Timer
    if (WDTE) {
      if (time >= UpdateTime2) {
        //Increment WDT
        if (!PSA) WDTCounter++;
        else {
          Prescaler+=(word)(0x100>>(PS012));
          if (Prescaler&0x100) { WDTCounter++;Prescaler=0; }
        }
        //WDT counter overflow test
        if (WDTCounter&0x100) {
          // Faire un Reset sur cause WDT
          CycleCount=3;
          Reset(RST_Timeout);
        }
        UpdateTime2 += WDTPERIOD;
      }
    } else UpdateTime2=+INFINITY;
    if (!PowerDown) {
      //internal oscillator
      if (IRC) {
        if (time >= UpdateTime1)
        {
          SpecialPins^=0x10;
          UpdateTime1 += IClkPeriod;
        }
        SpecialPins=(byte)((SpecialPins&0xFE)+((SpecialPins>>4)&1));
      } else UpdateTime1=+INFINITY;
      //CPU
      if ((LastSpecialPins&1)&&(!(SpecialPins&1)))
      {
        if (TURBO) SystemClock=4; else SystemClock++;
        if (SystemClock==4) {
          SystemClock=0;
          //Gestion de RTCC
          if (RTS) {
            //RTCC incrment par signaux extrieurs
            if (RTCC.Detect) {
              if (PSA) RTCCCounter++;
              else {
                Prescaler+=(word)(128>>(PS012));
                if (Prescaler&0x100) { RTCCCounter++;Prescaler=0; }
              }
            }
          } else {
            //RTCC incrment par program counter
            if (PSA) RTCCCounter++;
            else {
              Prescaler+=(word)(128>>(PS012));
              if (Prescaler&0x100) { RTCCCounter++;Prescaler=0; }
            }
          }
          if (RTCCCounter==0x100) {
            RTCCCounter=0;
            //Tester si l'interruption sur roll-over est active
            if (!RTE_IE && !IntStack.IntActive) {
              //Activer l'interruption
              //1:sauvegarde des registres
              IntStack.W=W;
              IntStack.FSR=fr_FSR;
              IntStack.STATUS=fr_STATUS;
              IntStack.PC=PC;
              IntStack.IntActive = 1;
	          fr_STATUS &= 0x1F; // Clear PAGE
              //2:branchement
              CycleCount=3;
              PC=0;
              TestBreakPoints();
            }
          }
          RTCC.Detect=0;
          if (!CycleCount) {
            if (LastStepID!=Project.GetStepID()) {
              StepStackPtr=SP;
              LastStepID=Project.GetStepID();
            }
            CycleCount=Execute();
            TestBreakPoints();
          }
          CycleCount--;
        }
      }
    } else UpdateTime1=+INFINITY;
    updatetime=MIN(UpdateTime1,UpdateTime2);
  } else {
    //Le reset est activ: remettre  zro le systme
    if (LastSpecialPins&0x04) Reset(RST_MCLR);
    updatetime=+INFINITY;
  }
  LastSpecialPins=SpecialPins;
  updateneeded=0;
}

void Sx1828::TestBreakPoints()
{
  if (SP<=StepStackPtr) Project.Step();
  Project.Trace();
  if (ToCursorBreakAddr == PC || BreakPoints[PC]) {
    //Handle breakpoint
    ToCursorBreakAddr = -1;
    Project.Break();
  }
}

void Sx1828::Reset(int type)
{
  LastStepID=Project.GetStepID();
  if (type&(RST_Powerup|RST_Brownout)) {
    W=(byte)random(256);
    RTCCCounter=(byte)random(256);
    //Remplir alatoirement la mmoire
    for (int t=5;t<RamSize;t++) RAM[t]=(byte)random(256);
    RAM[5] &= 0xF;
    WKPND_B=(byte)random(256);
    WDTCounter=(byte)random(256);
  }
  OPTION=0xFF;
  MODE=0x0F;
  PC=(word)(RomSize-1);
  SP=0;
  fr_STATUS=(byte)(fr_STATUS&0x1F); //bits 5..7 STATUS=0;
  IntStack.IntActive = 0; // Is this true?? #### 
  switch (type) {
    case RST_Powerup: {
      fr_STATUS=(byte)(random(8)+0x18);
      fr_FSR=(byte)(random(256));
      break;
    }
    case RST_Wakeup: {
      fr_STATUS=(byte)((fr_STATUS&0x18)+random(8));
      fr_FSR=(byte)(random(256)|0x80);
      break;
    }
    case RST_Brownout: {
      fr_STATUS=(byte)(random(32));
      fr_FSR=(byte)(random(256)|0x80);
    }
    case RST_Timeout: {
      if (PowerDown) fr_STATUS=(byte)(fr_STATUS&0x7);
        else fr_STATUS=(byte)((fr_STATUS&0x7)+0x08);
      fr_FSR|=0x80;
      break;
    }
    case RST_MCLR: {
      fr_STATUS=(byte)(fr_STATUS&0x1F);
      if (PowerDown) fr_STATUS=(byte)((fr_STATUS&0x7)+0x10);
      fr_FSR|=0x80;
      break;
    }
  }
  for (int t=0;t<3;t++) { TRIS[t]=LVL[t]=PLP[t]=0xFF; }
  CMP_B=(byte)(random(256)|0xC1);
  WKED_B=0xFF;
  WKEN_B=0xFF;
  ST_B=ST_C=0xFF;
  PowerDown=false;
  //Initiate special vars
  Fuse=ROM[0x1010];
  Fusex=ROM[0x1011];
  switch ((Fuse>>5)&3) {
    case 0:{ IClkPeriod=0.5/4E6;break;}
    case 1:{ IClkPeriod=0.5/1E6;break;}
    case 2:{ IClkPeriod=0.5/128E3;break;}
    case 3:{ IClkPeriod=0.5/32E3;break;}
  }
  if (STACK_X) StackDepth=2; else StackDepth=8;
  //
  RAM[0]=RAM[fr_FSR];
}

//$$$$ tester le cas n=0,FSR=0 sur le SX
unsigned char Sx1828::Getfr(int n)
{
  n = (n & 0x1F) | (fr_FSR & 0xE0);
  if (!(n & 0x1F)) n=fr_FSR;
  if (!(n&0x10)) {
    n&=0xF;
    switch (n) {
      case 1:{
        if (RTW) return (byte)RTCCCounter; else return W;
      }
      case 2:{
        return (byte)(PC&0xFF);
      }
      case 5:
	return RAM[5] & 0x0F;
      default:
        return RAM[n];
    }
  }
  return RAM[n];
}

//modifie un registre
void Sx1828::Setfr(int n,unsigned char v)
{
  n = (n & 0x1F) | (fr_FSR & 0xE0);
  if (!(n & 0x1F)) {
    n=fr_FSR;
    RAM[0]=v;
  }
  if (!(n&0x10)) {
    n&=0xF;
    switch (n) {
      case 1:{
        if (RTW) {
          RTCCCounter=v;
          Prescaler=0; //a tester : non document
        } else W=v;
        break;
      }
      case 2:{  //PC
        PC=(word)(PC&0xF00+v);
        break;
      }
      case 5:
      case 6:
      case 7:{
        R_OUT[n-5]=v;
        break;
      }
    }
  }
  RAM[n]=v;
}

void Sx1828::ModifyPortConfig(byte reg) {
  if ((reg>=5)&&(reg<=7))  {
    switch (MODE) {
      case 0x0C:{
        if (reg==6) ST_B=W;
        else if (reg==7) ST_C=W;
        break;
      }
      case 0x0D:{
        LVL[reg-5]=W;
        if (reg==5)
          for (int t=0;t<4;t++) PORT[reg-5][t]->SetTTL((W>>t)&1);
        else
          for (int t=0;t<8;t++) PORT[reg-5][t]->SetTTL((W>>t)&1);
        break;
      }
      case 0x0E:{PLP[reg-5]=W;break;}
      case 0x0F:{TRIS[reg-5]=W;break;}
    }
  }
  byte temp;
  if (reg==6) switch (MODE) {
    case 0x08:{temp=W;W=CMP_B;CMP_B=temp;break;}
    case 0X09:{temp=W;W=WKPND_B;WKPND_B=temp;break;}
    case 0x0A:{WKED_B=W;break;}
    case 0x0B:{WKEN_B=W;break;}
  }
}

void Sx1828::PushStack(word aPC)
{
  SP++;
  if (SP==StackDepth) 
  {
    error(TYPE_WARNING,-1,NULL,"SX_Stack overflow in %s (level %d)",getfullnametext(),SP);
    Project.Break();
  }
  memmove(&Stack[1],&Stack[0],(StackDepth-1)*(sizeof (word)));
  Stack[0]=aPC;
}

word Sx1828::PopStack(void)
{
  word temp;
  temp=Stack[0];
  memmove(&Stack[0],&Stack[1],(StackDepth-1)*(sizeof (word)));
  if (!SP)
  {
    error(TYPE_WARNING,-1,NULL,"SX_Stack underflow in %s (level %d)",getfullnametext(),SP);
    Project.Break();
  }
  SP--;
  return temp;
}

// Traduit opCode en assembleur
char * Sx1828::Disasm(word Addr)
{
  int type; //type de paramtres pour les instructions.
  word Code=ROM[Addr];
  static char data[100];
  char *buffer=(char*)(&data);
  char *temp="";
  //Instructions sans paramtres.
  switch (Code) {
    case CLR_W   : { buffer="CLR W";break;}
    case CLR_WDT : { buffer="CLR WDT";break;}
    case MOV_WM  : { buffer="MOV W,M";break;}
    case MOV_MW  : { buffer="MOV M,W";break;}
    case MOV_OPTIONW : { buffer="MOV !OPTION,W";break;}
    case NOP     : { buffer="NOP";break;}
    case RET     : { buffer="RET";break;}
    case RETP    : { buffer="RETP";break;}
    case RETI    : { buffer="RETI";break;}
    case RETIW   : { buffer="RETIW";break;}
    case IREAD   : { buffer="IREAD";break;}
    case SLEEP   : { buffer="SLEEP";break;}
    default :
    {
      //Instructions avec 3 bits de paramtres.
      switch (Code&0xFF8) {
        case BANK_ad8 : { sprintf(buffer,"BANK  %d",Code&0x007);break;}
        case PAGE     : { sprintf(buffer,"PAGE  %d",Code&0x007);break;}
        default :
        {
          //Instructions avec 4 bits de paramtres.
          switch (Code&0xFF0) {
            case MOV_rxW  : { sprintf(buffer,"MOV   !%s,W",Register((byte)(Code&0x00F)));break; }
            case MOV_Mlit : { sprintf(buffer,"MOV   M,0x%0.2X",Code&0x00F);break; }
            default :
            {
              type=0;
              //Instructions avec 5 bits de paramtres.
              switch (Code&0xFE0) {
                case AND_frW     : { temp="AND   %s,W";break;}
                case AND_Wfr     : { temp="AND   W,%s";break;}
                case CLR_fr      : { temp="CLR   %s";break;}
                case NOT_fr      : { temp="NOT   %s";break;}
                case OR_frW      : { temp="OR    %s,W";break;}
                case OR_Wfr      : { temp="OR    W,%s";break;}
                case XOR_frW     : { temp="XOR   %s,W";break;}
                case XOR_Wfr     : { temp="XOR   W,%s";break;}
                case ADD_frW     : { temp="ADD   %s,W";break;}
                case ADD_Wfr     : { temp="ADD   W,%s";break;}
                case DEC_fr      : { temp="DEC   %s";break;}
                case DECSZ_fr    : { temp="DECSZ %s";break;}
                case INC_fr      : { temp="INC   %s";break;}
                case INCSZ_fr    : { temp="INCSZ %s";break;}
                case RL_fr       : { temp="RL    %s";break;}
                case RR_fr       : { temp="RR    %s";break;}
                case SUB_frW     : { temp="SUB   %s,W";break;}
                case SWAP_fr     : { temp="SWAP  %s";break;}
                case MOV_frW     : { temp="MOV   %s,W";break;}
                case MOV_Wfr     : { temp="MOV   W,%s";break;}
                case MOV_Wfr_W   : { temp="MOV   W,%s-W";break;}
                case MOV_Wnotfr  : { temp="MOV   W,not(%s)";break;}
                case MOV_Wdecfr  : { temp="MOV   W,%s-1";break;}
                case MOV_Wincfr  : { temp="MOV   W,%s+1";break;}
                case MOV_Wleftfr : { temp="MOV   W,%s shl 1";break;}
                case MOV_Wrightfr: { temp="MOV   W,%s shr 1";break;}
                case MOV_Wswapfr : { temp="MOV   W,swap(%s)";break;}
                case MOVSZ_Wdecfr: { temp="MOVSZ W,%s-1";break;}
                case MOVSZ_Wincfr: { temp="MOVSZ W,%s+1";break;}
                case TEST_fr     : { temp="TEST  %s";break;}
                default: {
                  type=1;
                  //Cas des instructions avec 8 bits de paramtres.
                  switch (Code&0xF00) {
                    case AND_Wlit : { temp="AND   W,0x%0.2X";break;}
                    case OR_Wlit  : { temp="OR    W,0x%0.2X";break;}
                    case XOR_Wlit : { temp="XOR   W,0x%0.2X";break;}
                    case MOV_Wlit : { temp="MOV   W,0x%0.2X";break;}
                    case CALL_ad8 : { temp="CALL  0x%0.2X";break;}
                    case RETW_lit : { temp="RETW  0x%0.2X";break;}
                    case CLRB_frb : { temp="CLRB  %s";type=2;break;}
                    case SB_frb   : { temp="SB    %s";type=2;break;}
                    case SETB_frb : { temp="SETB  %s";type=2;break;}
                    case SNB_frb  : { temp="SNB   %s";type=2;break;}
                    default:{
                      if ((Code&0xE00)==JMP_ad9) {
                        temp="JMP   %0.4X";
                        type=3;
                      } else
                      {
                        temp="Unknown";
                      }
                    }
                  }
                }
              } //
              switch (type) {
                case 0 : { sprintf(buffer,temp,Register((byte)(Code&0x01F))); break; }
                case 1 : { sprintf(buffer,temp,Code&0x0FF); break; }
                case 2 : { sprintf(buffer,temp,Bits((byte)(Code&0x0FF))); break; }
                case 3 : { sprintf(buffer,temp,Code&0x1FF); break; }
              }
            } //end default
          } //end switch (Code&0xFF0)==0x000)
        } //end default
      } //end switch (Code&0xFF8)
    }  // end default
  }  //end switch (Code)

  return buffer;
}

//******************************************************************************

// Dfinition de PC  revoir
// Traduit opCode en assembleur
int Sx1828::Execute(void)
{
  word Code=ROM[PC];
  int cycles=1;  //Par dfaut 1 instruction=1 cycle.
  int jmpcycle;
  //nb de cycles pour une instruction de branchement
  if (TURBO) jmpcycle=3; else jmpcycle=2;
  //Incrmente le pointeur programme
  PC=(word)((PC+1)%RomSize);
  //Instructions sans paramtres.
  switch (Code) {
    case CLR_W   : { W=0;fr_STATUS|=STATUS_Z;break; }
    case CLR_WDT : { WDT=0;WDT_PRE=0;fr_STATUS|=STATUS_Z+STATUS_TO+STATUS_PD;break;}
    case MOV_WM  : { W=MODE;break;}
    case MOV_MW  : { MODE=W;break;}
    case MOV_OPTIONW : {
      OPTION=W;
      if (OPTION_X) OPTION|=0xC0;
      RTCC.Edge=(byte)(RTE_ES);
      break;
    }
    case NOP     : { break;}
    case RET     : { PC=PopStack();cycles=jmpcycle; break;}
    case RETP    : {
      PC=PopStack();
      fr_STATUS=(byte)((fr_STATUS&0x1F)|((PC&0xE00)>>4));
      cycles=jmpcycle;
      break;
      }
    case RETIW   : {
      fr_RTCC+=W;
      //break volontairement supprim.
    }
    case RETI    : {
      if (!IntStack.IntActive)
      {
	error(TYPE_WARNING, -1, NULL, "RETI while not in interrupt!");
	Project.Break();
      }
      W=IntStack.W;
      fr_FSR=IntStack.FSR;
      fr_STATUS=IntStack.STATUS;
      PC=IntStack.PC;
      cycles=jmpcycle;
      IntStack.IntActive = 0;
      break;
    }
    case IREAD   : {
      word temp;
      temp=(word)((((word)MODE&0x0F)<<8)|(word)W);
      if (temp<RomSize) temp=ROM[temp];
      MODE=(byte)(temp>>8);
      W=(byte)(temp&0xFF);
      if (TURBO) cycles=4;
      break;
    }
    case SLEEP   : {
      PowerDown=true;
      CHANGE_PD(0);
      CHANGE_TO(1);
      break;
    }
    default :
    {
      //Instructions avec 3 bits de paramtres.
      switch (Code&0xFF8) {
        case BANK_ad8 : { fr_FSR=(byte)((fr_FSR&0x1F)|((Code&0x007)<<5));break;}
        case PAGE     : { fr_STATUS=(byte)((fr_STATUS&0x1F)|((Code&0x007)<<5));break;}
        default :
        {
          //Instructions avec 4 bits de paramtres.
          switch (Code&0xFF0) {
            case MOV_rxW  : {
              ModifyPortConfig((byte)(Code&0x0F));
              break;
            }
            case MOV_Mlit : { MODE=(byte)(Code&0x00F);break; }
            default :
            {
              char adfr=(byte)(Code&0x1F);
              int fr=Getfr(adfr);
              int temp;
              //Instructions avec 5 bits de paramtres.
              switch (Code&0xFE0) {
                case AND_frW:
                case AND_Wfr : {
                  temp=fr&W;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==AND_frW) Setfr(adfr,(byte)temp);else W=(byte)temp;
                  break;
                }
                case CLR_fr:
                {
                  Setfr(adfr,0);
                  CHANGE_Z(1);
                  break;
                }
                case NOT_fr:
                case MOV_Wnotfr : {
                  temp=fr^0xFF;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==NOT_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case OR_frW:
                case OR_Wfr : {
                  temp=fr|W;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==OR_frW) Setfr(adfr,(byte)temp);else W=(byte)temp;
                  break;
                }
                case XOR_frW:
                case XOR_Wfr : {
                  temp=fr^W;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==XOR_frW) Setfr(adfr,(byte)temp);else W=(byte)temp;
                  break;}
                case ADD_frW:
                case ADD_Wfr : {
                  temp=fr+(int)W;
                  if ((!CONFIG_CF)&&(GET_C)) temp++;
                  CHANGE_DC((((fr^W)^temp)>>4)&1);
                  CHANGE_C(temp>>8);
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==ADD_frW) Setfr(adfr,(byte)temp);else W=(byte)temp;
                  break;
                }
                case DEC_fr:
                case MOV_Wdecfr : {
                  temp=fr-1;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==DEC_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case DECSZ_fr:
                case MOVSZ_Wdecfr : {
                  temp=fr-1;
                  if (!(temp&0xFF)) { PC++;cycles=2; }
                  if ((Code&0xFE0)==DECSZ_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case INC_fr :
                case MOV_Wincfr : {
                  temp=fr+1;
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==INC_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case INCSZ_fr:
                case MOVSZ_Wincfr : {
                  temp=fr+1;
                  if (!(temp&0xFF)) { PC++;cycles=2; }
                  if ((Code&0xFE0)==INCSZ_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case RL_fr:
                case MOV_Wleftfr : {
                  temp=(fr<<1)|(GET_C);
                  CHANGE_C(temp>>8);
                  if ((Code&0xFE0)==RL_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case RR_fr:
                case MOV_Wrightfr : {
                  temp=fr|(GET_C<<8);
                  CHANGE_C(temp&1);
                  temp=temp>>1;
                  if ((Code&0xFE0)==RR_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case SUB_frW:
                case MOV_Wfr_W : {
                  temp=fr-(int)W;
                  if ((!CONFIG_CF)&&(!(GET_C))) temp--;
                  CHANGE_DC(1-((((fr^W)^temp)>>4)&1));
                  CHANGE_C(1-((temp&0x100)>>8));
                  if (temp&0xFF) CHANGE_Z(0); else CHANGE_Z(1);
                  if ((Code&0xFE0)==SUB_frW) Setfr(adfr,(byte)temp);else W=(byte)temp;
                  break;
                }
                case SWAP_fr:
                case MOV_Wswapfr : {
                  temp=((temp&0xF0)>>4)|((temp&0x0F)<<4);
                  if ((Code&0xFE0)==SWAP_fr) Setfr(adfr,(byte)temp); else W=(byte)temp;
                  break;
                }
                case MOV_frW     : {
                  Setfr(adfr,W);
                  break;
                }
                case MOV_Wfr     : {
                  W=(byte)fr;
                  if (fr) CHANGE_Z(0); else CHANGE_Z(1);
                  break;
                }
                case TEST_fr     : {
                  if (fr) CHANGE_Z(0); else CHANGE_Z(1);
                  break;
                }

                default: {
                  //Cas des instructions avec 8 bits de paramtres.
                  byte b=(byte)(1<<((byte)((Code>>5)&7)));
                  switch (Code&0xF00) {
                    case AND_Wlit : {
                      W&=(byte)(Code&0xFF);
                      if (W) CHANGE_Z(0); else CHANGE_Z(1);
                      break;
                    }
                    case OR_Wlit  : {
                      W|=(byte)(Code&0xFF);
                      if (W) CHANGE_Z(0); else CHANGE_Z(1);
                      break;
                    }
                    case XOR_Wlit : {
                      W^=(byte)(Code&0xFF);
                      if (W) CHANGE_Z(0); else CHANGE_Z(1);
                      break;
                    }
                    case MOV_Wlit : {
                      W=(byte)(Code&0xFF);
                      break;
                    }
                    case CALL_ad8 : {
                      PushStack(PC);
                      PC=(word)((Code&0xFF)+((fr_STATUS&0xE0)<<4));
                      cycles=jmpcycle;
                      break;
                    }
                    case RETW_lit : {
                      PC=PopStack();
                      W=(byte)(Code&0xFF);
                      cycles=jmpcycle;
                      break;
                    }
                    case CLRB_frb : {
                      Setfr(adfr,(byte)(fr&(~b)));
                      break;
                    }
                    case SB_frb   : {
                      if (fr&b) { PC++;cycles=2; }
                      break;
                    }
                    case SETB_frb : {
                      Setfr(adfr,(byte)(fr|b));
                      break;
                    }
                    case SNB_frb  : {
                      if (!(fr&b)) { PC++;cycles=2; }
                      break;
                    }
                    default:{
                      if ((Code&0xE00)==JMP_ad9) {
                        PC=(word)((PC&0xE00)+(Code&0x1FF));
                        cycles=jmpcycle;
                      } else
                      {
                        //Erreur opcode inconnu;
                        // Vrifier si c'est possible
                      }
                    }
                  }
                }
              } //
            } //end default
          } //end switch (Code&0xFF0)==0x000)
        } //end default
      } //end switch (Code&0xFF8)
    }  // end default
  }  //end switch (Code)

  return cycles;
}


int Sx1828::LoadHexFile(char* FileName)
{
  return HexRead(FileName,(char*)ROM,2*0x1012);
}

int Sx1828::LoadLstFile(char* FileName)
{
  return LstRead(FileName,(short*)ROM,Lines,SourceLines,SourceToAddr,AddrToSource,ROM[0x1010],ROM[0x1011],RomSize, NumLines);
}

Sx1828::~Sx1828(void)
{
  delete RAM;
  delete ROM;
}

//---------------------------------------------------------------------------
#ifndef __linux__
#pragma package(smart_init)
#endif
