/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __ATTRIBUTE_H__
#define __ATTRIBUTE_H__

#define nullattribute _nullattribute()

class _attribute;

class attribute
{
  friend _attribute;
  friend class _pointerattribute;
  friend attribute *_nullattribute();
protected:
  attribute() {};

  virtual int getstringattribute(char **val) { return 0; }
  virtual int getintattribute(int *i) { return 0; }
  virtual int getnameattribute(symbolname **s) { return 0; }
  virtual int getrealattribute(real *r) { return 0; }
public:

  int getstringattribute(int name, char **val, int maxpos = 0);
  int getintattribute(int name, int *i, int maxpos = 0);
  int getnameattribute(int name, symbolname **s, int maxpos = 0);
  int getrealattribute(int name, real *r, int maxpos = 0);

  int getstringattribute(char *name, char **val, int maxpos = 0) { return getstringattribute(symbtable.toid(name), val, maxpos); }
  int getintattribute(char *name, int *i, int maxpos = 0) { return getintattribute(symbtable.toid(name), i, maxpos); }
  int getnameattribute(char *name, symbolname **s, int maxpos = 0) { return getnameattribute(symbtable.toid(name), s, maxpos); }
  int getrealattribute(char *name, real *r, int maxpos = 0) { return getrealattribute(symbtable.toid(name), r, maxpos); }

  virtual attribute *addattribstoend(attribute *attribs) { return attribs; }
  virtual attribute *getattribute(int name, int *pos = (int *) NULL) { if (pos != NULL) *pos = 0; return nullattribute; }
  //  virtual attribute *changeattribute(_attribute *newattrib);
  virtual attribute *addattribute(_attribute *newattrib);
  virtual void destroy() {}
};

class _attribute : public attribute
{
  friend inline attribute *paddingattribute() { return new _attribute(symbtable.toid("__paddingattribute__")); }
private:
  int name;
  attribute *next;

protected:
  _attribute(int name, attribute *next = nullattribute);
  virtual ~_attribute() { if (next != nullattribute) delete next; }

public:
  attribute *addattribstoend(attribute *attribs) { if (next != nullattribute) next = next->addattribstoend(attribs); else next = attribs; return this; }
  attribute *getattribute(int name, int *pos = (int *) NULL);
  //  attribute *changeattribute(_attribute *newattrib);
  attribute *addattribute(_attribute *newattrib);
  virtual void destroy() { delete this; }
};

class _emptyattribute : public _attribute
{
  friend _emptyattribute *emptyattribute(int name, attribute *next = nullattribute) { return new _emptyattribute(name, next); };

  _emptyattribute(int name, attribute *next=(attribute*) NULL);    
};

class _intattribute : public _attribute
{
  friend _intattribute *intattribute(int name, int val, attribute *next = nullattribute) { return new _intattribute(name, val, next); };

  int val;
  _intattribute(int name, int val, attribute *next=(attribute*) NULL);    //FM

protected:
  int getintattribute(int *i) { *i = val; return 1; }
  int getrealattribute(real *r) { *r = val; return 1; }
};

class _stringattribute : public _attribute
{
  friend _stringattribute *stringattribute(int name, char *val, attribute *next = nullattribute) { return new _stringattribute(name, val, next); };

  char *val;
  _stringattribute(int name, char *val, attribute *next=(attribute*) NULL);    //FM

protected:
  virtual int getstringattribute(char **pval) { *pval = val; return 1; }

public:
  void destroy() { free(val); }
};

class _realattribute : public _attribute
{
  friend _realattribute *realattribute(int name, real val, attribute *next = nullattribute) { return new _realattribute(name, val, next); }

  real val;
  _realattribute(int name, real val, attribute *next=(attribute*) NULL);   //FM

protected:
  virtual int getrealattribute(real *r) { *r = val; return 1; }
};

class _nameattribute : public _attribute
{
  friend _nameattribute *nameattribute(int name, symbolname *val, attribute *next = nullattribute) { return new _nameattribute(name, val, next); }

  symbolname *val;
  _nameattribute(int name, symbolname *val, attribute *next=(attribute*) NULL);    //FM
  virtual ~_nameattribute() { delete val; };

protected:
  int getnameattribute(symbolname **s) { *s = val; return 1; }
};

class _pointerattribute : public _attribute
{
  friend _pointerattribute *pointerattribute(int name, int attribname, attribute *next = nullattribute) { return new _pointerattribute(name, attribname, next); }

  int attribname;

  _pointerattribute(int name, int attribname, attribute *next=(attribute*) NULL);  //FM

  // #### Warn if invalid pointer
protected:
  int getintattribute(int *i) { return getattribute(attribname)->getintattribute(i); }
  int getnameattribute(symbolname **s) { return getattribute(attribname)->getnameattribute(s);  }
  int getrealattribute(real *r) { return getattribute(attribname)->getrealattribute(r); }
  int getstringattribute(char **s) { return getattribute(attribname)->getstringattribute(s); }
};

#endif
