/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#ifndef basecomponentsH
#define basecomponentsH
#ifndef __PROJECT_H__
  #include "project.h"
#endif

/*******************************************************************************
Composant NOT
*******************************************************************************/
class not : public component
{
protected:
  incomppin A;
  outpin O;
public:
  virtual pin *ToPin() { return &O; }
  virtual void Update(int force = 0);
  not(componenttype *type, int name, attribute *attribs, symbol *parent);
};

typedef genericcomponenttype<not> nottype;

/*******************************************************************************
Composant AND
*******************************************************************************/
class and : public not
{
protected:
  incomppin B;
public:
  virtual void Update(int force = 0);
  and(componenttype *type, int name, attribute *attribs, symbol *parent);
};

typedef genericcomponenttype<and> andtype;

/*******************************************************************************
Composant OR
*******************************************************************************/
class or : public and
{
public:
  virtual void Update(int force = 0);
  or::or(componenttype *type, int name, attribute *attribs, symbol *parent)
  : and(type, name, attribs, parent) {}
};

typedef genericcomponenttype<or> ortype;

/*******************************************************************************
Composant XOR
*******************************************************************************/
class xor : public and
{
public:
  virtual void Update(int force = 0);
  xor::xor(componenttype *type, int name, attribute *attribs, symbol *parent)
  : and(type, name, attribs, parent) {}
};

typedef genericcomponenttype<xor> xortype;

/*******************************************************************************
Composant NAND
*******************************************************************************/
class nand : public and
{
public:
  virtual void Update(int force = 0);
  nand::nand(componenttype *type, int name, attribute *attribs, symbol *parent)
  : and(type, name, attribs, parent) {}
};

typedef genericcomponenttype<nand> nandtype;

/*******************************************************************************
Composant NOR
*******************************************************************************/
class nor : public and
{
public:
  virtual void Update(int force = 0);
  nor::nor(componenttype *type, int name, attribute *attribs, symbol *parent)
  : and(type, name, attribs, parent) {}
};

typedef genericcomponenttype<nor> nortype;

/*******************************************************************************
Composant XNOR
*******************************************************************************/
class xnor : public and
{
public:
  virtual void Update(int force = 0);
  xnor::xnor(componenttype *type, int name, attribute *attribs, symbol *parent)
  : and(type, name, attribs, parent) {}
};

typedef genericcomponenttype<xnor> xnortype;

class powerdc : public component
{
  outpin O;

public:
  virtual pin *ToPin() { return &O; }
  virtual void parseattribs(attribute *attribs);
  virtual void Update(int force = 0);   
  powerdc(componenttype *type, int name, attribute *attribs, symbol *parent);
};

typedef genericcomponenttype<powerdc> powerdctype ;

class Clock : public component
{
  real T;
  outpin O;

  symbol *_getlocalsymbol(list<int> *name);
public:
  virtual void parseattribs(attribute *attribs);
  virtual void Update(int force = 0);
  void _Reset();
  Clock(componenttype *type, int name, attribute *attribs, symbol *parent);
  virtual pin *ToPin() { return &O; }
  list<symbol> *getapparentsymbols();
};

typedef genericcomponenttype<Clock> clocktype;

class flipflop : public component
{
  incomppin S, R;
  outpin Q, NQ;
  int state;
  basevariableextract<int> svar;

public:
  virtual void Update(int force = 0);
  flipflop(componenttype *type, int name, attribute *attribs, symbol *parent);
  virtual pin *ToPin() { return &Q; }
};

typedef genericcomponenttype<flipflop> flipfloptype;

class motor : public component
{
  outpin quad1, quad2;
  incomppin pwm, dir;
  real x, v, a, extforce;
  real quadupdatetime, simulupdatetime;
  real A, B, C, timestep; // accel = A * pwm - B * vel and quadticks = x * C

  symbol *_getlocalsymbol(list<int> *name);
public:
  void _Reset();
  virtual void Update(int force = 0);
  motor(componenttype *type, int name, attribute *attribs, symbol *parent);
  list<symbol> *getapparentsymbols();
  void parseattribs(attribute *attribs);
};

typedef genericcomponenttype<motor> motortype;

//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
#endif

