/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

void RegisterComponents()
{
  RegisterComponentType(new powerdctype, "powerdc");
  RegisterComponentType(new clocktype, "clock");
  RegisterComponentType(new nandtype, "nand");
  RegisterComponentType(new flipfloptype, "flipflop");
  RegisterComponentType(new Sx1828type, "sx18");
  RegisterComponentType(new Sx1828type, "sx28");
  RegisterComponentType(new nottype, "not");
  RegisterComponentType(new andtype, "and");
  RegisterComponentType(new ortype, "or");
  RegisterComponentType(new xortype, "xor");
  RegisterComponentType(new nandtype, "nand");
  RegisterComponentType(new nortype, "nor");
  RegisterComponentType(new xnortype, "xnor");
  RegisterComponentType(new Oscillotype,"oscillo");
  RegisterComponentType(new motortype,"motor");
}

componenttype::componenttype()
{
  name = -1;
}

macrocomponenttype::macrocomponenttype()
{
  symbolconstructors = (list<symbolconstructor> *) NULL;
  attribs = nullattribute;
}

macrocomponenttype::~macrocomponenttype()
{
  for (list<symbolconstructor> *l = symbolconstructors; l != NULL; l = l->next)
    delete l->elem;
  delete symbolconstructors;
}

void macrocomponenttype::AddSymbol(symbolconstructor *s)
{
  symbolconstructors = new list<symbolconstructor>(s, symbolconstructors);  
  s->addattribstoend(attribs);
}

void macrocomponenttype::AddComponent(int name, list<componentconstructor> *cc)
{
  componenttype *ct = Project.GetComponentType(name);

  if (ct == NULL)
  {
    parseerror(TYPE_ERROR, "Components not created because type was invalid.");
    return;
  }

  for (list<componentconstructor> *l = cc; l != NULL; l = l->next)
  {
    l->elem->getparent()->symbolconstructors = new list<symbolconstructor>(l->elem, l->elem->getparent()->symbolconstructors); 
    l->elem->type = ct;
    l->elem->addattribstoend(attribs);
  }

  delete cc;
}

component *macrocomponenttype::create(int name, attribute *attribs, symbol *parent)
{
  macrocomponent *c = new macrocomponent(this, name, attribs, parent);
  
  for (list<symbolconstructor> *l = symbolconstructors; l != NULL; l = l->next)
  { 
    l->elem->ccreate(c);
  }

  return c;
}




