/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

symbol *displayboxconstructor::_create(symbol *parent)
{
  return new displaybox(name, attribs, parent, this->type, this->constructors);
}

void displayboxconstructor::revconstructors()
{
  revlist(constructors);
}

void displayboxconstructor::addconstructor(symbolconstructor *sc)
{ 
  constructors = new list<symbolconstructor>(sc, constructors); 
  sc->addattribstoend(attribs); 
}

symbol *displaybox::_getlocalsymbol(symbolname *s)
{
  for (list<symbolconstructor> *l = constructors; l != NULL; l = l-> next)
    if (l->elem->getname() == *(s->elem)) 
    {
      symbol *sy = l->elem->create(this);
      return sy->getlocalsymbol(s->next);
    }
    
  return (symbol *) NULL;
}

displaybox::displaybox(int name, attribute *attribs, symbol *parent, int boxtype, list<symbolconstructor> *constructors) : symbol(name, attribs, parent)
{
  displaybox::boxtype = boxtype;
  displaybox::constructors = constructors;
}

/*displaybox::displaybox(int name, attribute *attribs, symbol *parent, int boxtype, list<symbol> *children) : symbol(name, attribs, parent)
{
  displaybox::boxtype = boxtype;
  displaybox::constructors = (list <symbolconstructor> *) NULL;
  displaybox::children = children;
  }*/

displaybox *displaybox::getempty(int type)
{
  static displaybox hbox(symbtable.toid(""), nullattribute, (symbol *) NULL, BOX_H),
                    vbox(symbtable.toid(""), nullattribute, (symbol *) NULL, BOX_V);

  if (type == BOX_H) return &hbox;
  if (type == BOX_V) return &vbox;

  error(TYPE_FATALINTERNALERROR, -1, NULL, "Internal error : displaybox::getempty called with invalid type");
  return (displaybox *) NULL;
}

symbolwidget *displaybox::getwidget()
{
  if (boxtype == BOX_H) return new hboxwidget(getattributes(), this);
  if (boxtype == BOX_V) return new vboxwidget(getattributes(), this);

  error(TYPE_INTERNALERROR, -1, NULL, "Unknown box type %i", boxtype);
  return new vboxwidget(getattributes(), this);
}
