/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------

/***********************************************************************
Lecture de fichiers .Lst
***********************************************************************/

int LstRead(char *FileName,short *Data,char **Lines,char **&SourceLines, int *&SourceToAddr, int *&AddrToSource, short &Fuse,short &Fusex, int MaxLength, int &NumLines)
{
	FILE 	*stream;
        int CurLine = 0;;
	int		i,j,k;
	int		Code,Addr;
    char*   tmpstring;

	stream=fopen(FileName,"rb");
	if(!stream)	return(1);
    AddrToSource = new int[MaxLength];
    for (i = 0; i < MaxLength; i++)
      AddrToSource[i] = -1;
    // find file size
    fseek(stream,0,SEEK_END);
    int filesize=ftell(stream);
    char *filedata=new char[filesize];
    //read the whole file
    fseek(stream,0,SEEK_SET);
    fread(filedata,1,filesize,stream);
    //
    tmpstring=filedata;
    for (i=0;i<filesize;i++) {
      if ((filedata[i]=='\r')||(filedata[i]=='\n')) {
        filedata[i]=0;

	if ((j=atoi(tmpstring)) != CurLine + 1)
	{
          if (strcmp("Cross Reference", tmpstring) == 0) break;
          if (j> CurLine + 1) {
            SourceLines = (char **) realloc(SourceLines, sizeof(char *) * j);
	    SourceToAddr = (int *) realloc(SourceToAddr, sizeof(int *) * j);
            for (k=CurLine+1;k<j;k++) {
              if (SourceLines == NULL || SourceToAddr == NULL) error(TYPE_FATALERROR, CurLine + 1, FileName, "Out of memory while parsing %s", FileName);
              SourceLines[k] = "";
              SourceToAddr[k] = -1;
            }
            CurLine=j-1;
          } else {
            tmpstring=&filedata[i+1];
            continue;
          }
	}
	SourceLines = (char **) realloc(SourceLines, sizeof(char *) * (CurLine + 1));
	SourceToAddr = (int *) realloc(SourceToAddr, sizeof(int *) * (CurLine + 1));
	if (SourceLines == NULL || SourceToAddr == NULL) error(TYPE_FATALERROR, CurLine + 1, FileName, "Out of memory while parsing %s", FileName);
	SourceLines[CurLine] = tmpstring+20;
	SourceToAddr[CurLine] = -1;

        //new line to process
        if (strlen(tmpstring)>=17) {
          //there is data to process in this line
          if ( ((tmpstring[6]>='0')&&(tmpstring[6]<='9'))||((tmpstring[6]>='A')&&(tmpstring[6]<='F'))) {
            //code present
            //find the first word of the line
            for (j=20;tmpstring[j]==' '||tmpstring[j]==8;j++);
            //Check if it is a "device xxxxx" line
            if (j<=(int)strlen(tmpstring)-6
                 && toupper(tmpstring[j]) == 'D' 
                 && toupper(tmpstring[j+1]) == 'E' 
                 && toupper(tmpstring[j+2]) == 'V'
                 && toupper(tmpstring[j+3]) == 'I' 
                 && toupper(tmpstring[j+4]) == 'C' 
                 && toupper(tmpstring[j+5]) == 'E') {
              //Device found
              Fuse=HexRead16((char*)&tmpstring[6]);
              Fusex=HexRead16((char*)&tmpstring[12]);
            } else {
              Addr=HexRead16((char*)&tmpstring[6]);
              Code=HexRead16((char*)&tmpstring[12]);
              if ((Addr>=0)&&(Addr<MaxLength)) {
                Data[Addr]=Code;
                Lines[Addr]=&tmpstring[20];
                for (;Lines[Addr][0]==' ';Lines[Addr]=Lines[Addr]+1) {}; //skip spaces
		if (AddrToSource[Addr] != -1) error(TYPE_WARNING, CurLine + 1, FileName, "Address %X defined twice. First definition at line %i.", Addr, AddrToSource[Addr]);
		AddrToSource[Addr] = CurLine;
		SourceToAddr[CurLine] = Addr;
              }
            }
          }
/*          if (tmpstring[7]=='-') {
            //label
          }
*/
        }
        //prepare the nextline
        tmpstring=&filedata[i+1];
    
        CurLine++;
      }
    }
    // close file
    fclose(stream);
    NumLines = CurLine;
    return(0);
}


//---------------------------------------------------------------------------
#ifndef __linux__
#pragma package(smart_init)
#endif
