 #include "project.h"

 int newline;

 void yyerror(char *Msg, ...);
 int yylex(void);
 #include "parser.tab.c"

 #define YY_USER_ACTION { if (newline > 0) newline--; }

 #define YY_ALWAYS_INTERACTIVE 0

 void parseerror(char *msg, ...);
 void parsefatalerror(char *msg, ...);
 int myyywrap();

 // %option noyywrap

%x 	cComment
%x 	cppComment
%x      String
%x      include
%x      includeend

ANY 		(.|"\n")
CHAR		[A-Za-z_]
NUM		[0-9]
ALPHANUM	[A-Za-z0-9_]

%%

"#include"[ \t]+\"      { BEGIN(include); if (!newline) { parseerror(TYPE_ERROR, "Misplaced #include!"); BEGIN(includeend); } }
<include>[^\"\n]+\"     { yytext[yyleng - 1] = 0; incrlinenum(); pushfile(yytext); }
<include>\n             { incrlinenum(); parseerror(TYPE_ERROR, "Unclosed string"); BEGIN(0); }
<includeend>[ \t]*\n    { BEGIN(0); incrlinenum(); }
<includeend>[^\n]       { parseerror(TYPE_ERROR, "Unexpected input after #include : %s", yytext); }

"/*" 			BEGIN(cComment);
<cComment>[^*\n] 
<cComment>"*"+"/"  	BEGIN(0);
<cComment>"*"*
<cComment>\n 		incrlinenum();

"//" 			BEGIN(cppComment);
<cppComment>.*"\n" 	{ incrlinenum(); BEGIN(0); }

"\""                    BEGIN(String);
<String>"\""            BEGIN(0);
<String>[^\"\n]*          { yylval.str = yytext; return STRING; }
<String>\n              { incrlinenum(); parseerror(TYPE_ERROR, "Unclosed string"); BEGIN(0); }

"\n"                    { incrlinenum(); }
"{" 			return LEFT_CURLY;
"}" 			return RIGHT_CURLY;
";" 			return SEMICOLON;
":" 			return COLON;
","			return COMMA;
"("			return LEFT_PAREN;
")"			return RIGHT_PAREN;
"<"			return LEFT_POINTY;
">"			return RIGHT_POINTY;
"/"			return SLASH;
"-"                     return MINUS;
"~"                     return TILDA;
"="                     return EQUAL;
"%"                     return PERCENT; 
"component"	       	return COMPONENT;
"net"                   return NET;
"symbol"                return SYMBOL;
"hbox"                  return HBOX;
"vbox"                  return VBOX;
"plug"                  return PLUG;
"var"                   return VAR;
"plugtype"              return PLUGTYPE;
"bus"                   { yylval.i = 0; return BUS; }
"mixedbus"              { yylval.i = 1; return BUS; }
[+-]?0x({NUM}|[A-Fa-f])+ { yylval.i = strtol(yytext, (char **) NULL, 16); return INTEGER; }
[+-]?$({NUM}|[A-Fa-f])+  { yylval.i = strtol(yytext + 1, (char **) NULL, 16); return INTEGER; }
[+-]?({NUM}|[A-Fa-f])+[Hh] { yylval.i = strtol(yytext, (char **) NULL, 16); return INTEGER; }
[+-]?({NUM}+"."{NUM}*|{NUM}*"."{NUM}+) { yylval.r = atof(yytext); return REAL; }
[+-]?{NUM}+             { yylval.i = atoi(yytext); return INTEGER; }
_                       { yylval.r = 1; return UNIT; }
_a                      { yylval.r = 1e-18; return UNIT; }
_f                      { yylval.r = 1e-15; return UNIT; }
_p                      { yylval.r = 1e-12; return UNIT; }
_n                      { yylval.r = 1e-9; return UNIT; }
_u                      { yylval.r = 1e-6; return UNIT; }
"_"                    { yylval.r = 1e-6; return UNIT; }
_m                      { yylval.r = 1e-3; return UNIT; }
_c                      { yylval.r = 1e-2; return UNIT; }
_d                      { yylval.r = 1e-1; return UNIT; }
_da                     { yylval.r = 1e1; return UNIT; }
_h                      { yylval.r = 1e2; return UNIT; }
_k                      { yylval.r = 1e3; return UNIT; }
_M                      { yylval.r = 1e6; return UNIT; }
_G                      { yylval.r = 1e9; return UNIT; }
_T                      { yylval.r = 1e12; return UNIT; }
_P                      { yylval.r = 1e15; return UNIT; }
{CHAR}{ALPHANUM}*	{ yylval.i = symbtable.toid(yytext); return NAME; }
[ \t\r]*
.			{ parseerror(TYPE_ERROR, "Invalid caracter : %s (%i)", yytext, yytext[0]); }
%%

class _yyfilestack
{
  int linenum;
  FILE *f;
  YY_BUFFER_STATE bs;
  static _yyfilestack *yyfilestack;
  _yyfilestack *next;
  char *filename;
  _yyfilestack(FILE *_f, char *file);
  ~_yyfilestack();
public:
  friend int emptystack();
  friend void pushfile(char *file);
  friend void pushfile(FILE *_f, char *file);
  friend int yywrap();
  friend void incrlinenum();
  friend int GetParseLineNum();
  friend char *GetParseFileName();
};

char *GetParseFileName() 
{ 
  if (_yyfilestack::yyfilestack != NULL) return _yyfilestack::yyfilestack->filename; 
  else return (char *) NULL; 
}

int GetParseLineNum() 
{ 
  if (_yyfilestack::yyfilestack != NULL) return _yyfilestack::yyfilestack->linenum; 
  else return -1; 
}

int yywrap() 
{ 
  delete _yyfilestack::yyfilestack; 
  return (_yyfilestack::yyfilestack == NULL); 
}

void pushfile(FILE *_f, char *file) 
{ 
  new _yyfilestack(_f, file); 
}

int emptystack() 
{ 
  return _yyfilestack::yyfilestack == NULL; 
}

void incrlinenum() 
{ 
  if (_yyfilestack::yyfilestack != NULL) 
    _yyfilestack::yyfilestack->linenum++; 
  newline = 2; 
}

class _yyfilestack *_yyfilestack::yyfilestack = (_yyfilestack *) NULL;

void pushfile(char *file)
{
  FILE *f = fopen(file, "r");
  if (f != NULL) new _yyfilestack(f, file);
  else 
  {
    parseerror(TYPE_ERROR, "Can't open %s", file);
    BEGIN(includeend);
  }
}

_yyfilestack::_yyfilestack(FILE *_f, char *file)
{ 
  DEBUG(cout << "Parsing file : " << file << "\n";)

  filename = strdup(file); 

  f = _f;
  linenum = 1;
  next = yyfilestack;
  yyfilestack = this; 
  bs = yy_new_buffer(f, YY_BUF_SIZE); 
  yy_switch_to_buffer(bs); 
  BEGIN(0);
}

_yyfilestack::~_yyfilestack() 
{ 
  DEBUG(cout << "Finished file : " << filename << "\n";)

  yyfilestack = next; 
  yy_delete_buffer(bs);
  fclose(f);
  BEGIN(includeend); 
  // free(filename); // Is used in symbolconstructor
  if (next != NULL) yy_switch_to_buffer(next->bs); 
}

void parse(FILE *infile, char *inname)
{
  pushfile(infile, inname);
  yyin = infile;
  yyparse();
  componenttype *maintype = Project.GetComponentType(symbtable.toid("main"));
  if (Project.maincomponent == NULL)
  {
    if (maintype == NULL) error(TYPE_WARNING, -1, inname, "No main component.");
    else Project.maincomponent = maintype->create(symbtable.toid("main"), nullattribute, &Project);
  }
  else if (Project.maincomponent != NULL) parseerror(TYPE_ERROR, "Main component redefined, old version kept.");
}

void parseerror(int type, char *msg, ...)
{
  va_list val;
  va_start(val, msg);

  vaerror(type, GetParseLineNum(), GetParseFileName(), msg, val);
}

void yyerror(char *Msg, ...)
{
  va_list val;
  va_start(val, Msg);
  
  vaerror(TYPE_ERROR, GetParseLineNum(), GetParseFileName(), Msg, val);
  error(TYPE_FATALERROR, GetParseLineNum(), GetParseFileName(), "Error was before \"%s\"", yytext);
}


