/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

project Project;
#ifdef __linux__
char *ProgramVersion = "xDust version 0.9 beta";
#else
char *ProgramVersion = "WinDust version 0.9 beta";
#endif

void RegisterComponentType(componenttype *ct, char *name)
{
  ct->name = symbtable.toid(name);
  Project.componenttypes = new list<componenttype>(ct, Project.componenttypes);
}

int project::IdleLoop()
{
  int retval;
  int doneunit = 0;
  int Loops = 0;

  //  cout << "Idle loop" << hex << RunMode << "\n";

 if (maincomponent != NULL) 
 {
   while (1)
   {
    switch (RunMode & RUN_MODEMASK)
    {
      case RUN_GO:
        Tick(MaxIter);
        retval=0;
        break;
      case RUN_STOP:
        retval = -1;
        RedrawAll();
        break;
      case RUN_ANIM:
        StepID++;
        TraceFlag = 0;
        StepFlag = 0;
        RunMode &= ~16;
      case RUN_CONTANIM:
        Tick(MaxIter);
        switch (RunMode & RUN_UNITMASK)
        {
          case RUN_TICK:
        doneunit = 1;
        break;
          case RUN_STEP:
        doneunit = StepFlag;
        break;
          case RUN_TRACE:
        doneunit = TraceFlag;
        break;
        }
        if (doneunit)
        {
      RunMode |= 16;
      switch (RunMode & RUN_ANIMMASK)
      {
        case RUN_FASTANIM:
          AnimateCount++;
          if (AnimateCount > FastAnimateSpeed)
          {
            RedrawAll();
            AnimateCount = 0;
          }
          retval = 0;
          break;
        case RUN_SLOWANIM:
          RedrawAll();
          retval = SlowAnimateDelay;
          break;
        case RUN_NORMANIM:
          RedrawAll();
          retval = 0;
          break;
        case RUN_ONCE:
          RunMode = RUN_STOP;
          RedrawAll();
          retval = -1;
          break;
      }
        }
        else retval = 0;
        break;
    }

    if (BreakFlag)
    {
      BreakFlag = 0;
      RunMode = RUN_STOP;
      RedrawAll();
      retval = -1;
    }

    if (Loops++ > MAXIDLEFUNCTIONLOOPS) break;
    if ((RunMode & RUN_MODEMASK) == RUN_GO) continue;
    if ((RunMode & (RUN_MODEMASK | RUN_ANIMMASK)) == (RUN_ANIM | RUN_FASTANIM)) continue;
    break;
   }
   return retval;
  } 
  else 
  {
    RedrawAll();
    RunMode=RUN_STOP;
    return -1;
  }
}

project::project() : symbol(symbtable.toid("__project__"), nullattribute, (symbol *) NULL),
  timesymbol(symbtable.toid("__time__"),nullattribute, this, &time),
  chronosymbol(symbtable.toid("__delay__"),nullattribute, this, &chrono)
  //, VCC(symbtable.toid("Vcc"), (component *) NULL, nullattribute, this, 1), GND(symbtable.toid("Vcc"), (component *) NULL, nullattribute, this)
{ 
  BreakFlag = 0;
  StepFlag = 0;
  TraceFlag = 0;
  symbols = (list<symbol> *) NULL; 
  componenttypes = (list<componenttype> *) NULL; 
  maincomponent = (component *) NULL; 
  time = chrono = lasttime = 0;
  windowrootconstructors = (list<symbolconstructor> *) NULL;
  windows = (list<window> *) NULL;
  plugtypes = (list<plugtype> *) NULL;
  //  TimeStep = TIMESTEP;
  AnimateCount = 0;
  FastAnimateSpeed = 10000;
  SlowAnimateDelay = 200;
  MaxIter = 100;
  RunMode = RUN_STOP;
  timesymbol.readonly=1;
}

void project::StartInterface()
{
  if (maincomponent) {
    maincomponent->Update(1); // Make sure everything is really initialized.

    for (list<symbolconstructor> *l = windowrootconstructors; l != NULL; l = l->next)
    {
      symbolwidget *s = l->elem->create(&Project)->getwidget();
      if (s == NULL)
      {
        l->elem->createerror(TYPE_ERROR, "Error creating window's contents.");
      }
      new window(getattributes(), s); // window automaticaly adds itself to the windows list and displays itself.
    }

    if (windows == NULL) new window(getattributes(), getwidget()); // No window is specified
  }
}

symbol *project::_getlocalsymbol(symbolname *s)
{
  if (maincomponent) {
    if (*s->elem == symbtable.toid("__project__")) return this->getlocalsymbol(s->next);

    if (*s->elem == symbtable.toid("main")) s = s->next;;
    return maincomponent->getlocalsymbol(s);
  }  

  return (symbol *) NULL;
}

void project::Reset()
{
  RunMode = RUN_STOP;
  time = lasttime = chrono = 0;
  maincomponent->Reset();
  RedrawAll();
}

void project::RedrawAll()
{
  if (maincomponent) {
    maincomponent->Update(1);
    for (list<window> *l = Project.windows; l != NULL; l = l->next)
      l->elem->refresh();
  }
}

void project::UnlinkWindow(window *w)
{
  if (windows == NULL)
  {
    error(TYPE_FATALERROR, -1, NULL, "project::UnlinkWindow called with windows == NULL");
  }

  if (windows->elem == w)
  {
    windows = windows->next; // #### Memory leak
    if (windows == NULL) QuitProgram();
    return;
  }

  for (list<window> *l = windows; l->next != NULL; l = l->next)
    if (l->next->elem == w)
    {
      l->next = l->next->next; // #### Memory leak
      return;
    }
}

int project::Tick(int maxstep)
{

  int i = 0;

  //  DEBUG(cerr << "\n\n\n*************************************";)

  time = /*MIN(*/maincomponent->UpdateTime()/*, time + TimeStep)*/;
  chrono += time-lasttime;
  lasttime = time;
  //  DEBUG(cerr << " Time " << time << " UT : " << maincomponent->UpdateTime() << "\n";)

  DispOptDebugInfo = 0; // ####

  do
  {
    i++;
    maincomponent->Update(1);
  } while (Project.maincomponent->UpdateNeeded() && i < maxstep);

  // DEBUG(cerr << i << " loops in Step\n";)

  if (i == maxstep)
  {
    error(TYPE_WARNING, -1, "Running", "Circuit didn't stabilize in %i passes.", i);
      // cout << i << " loops in Step\n";
    DispOptDebugInfo = 1; // ####
    maincomponent->Update(1);
    DispOptDebugInfo = 0; // ####
  }

  if (time == INFINITY)
  {
    error(TYPE_WARNING, -1, NULL, "Time has reached infinity. The big crunch is long past.");
    error(TYPE_WARNING, -1, NULL, "You've been working for a long time. Take a break and go home.");
    RunMode = RUN_STOP;
  }

  return i;
}

componenttype *project::GetComponentType(int name)
{
  for (list<componenttype> *l = componenttypes; l != NULL; l = l->next)
  {
    if (l->elem->name == name) return l->elem;
  }

  //  parseerror(TYPE_ERROR, "Unknown component type : %s", symbtable.tostring(name));

  return (componenttype *) NULL;
}

plugtype *project::GetPlugType(int name) 
{
  for (list<plugtype> *l = plugtypes; l != NULL; l = l->next)
  {
    if (l->elem->GetName() == name) return l->elem;
  }

  parseerror(TYPE_ERROR, "Unknown plugtype : %s", symbtable.tostring(name));
  
  return (plugtype *) NULL; 
}

ostream &operator<<(ostream &os, symbol &)
{
  return os;
}

template <class t>
ostream &operator<<(ostream &os, list<t> &_l)
{
  os << "List :\n";
  for (list<t> *l = &_l; l != NULL; l = l->next)
    os << l->elem << "\n";
  os << "ListEnd\n";
  return os;
}

ostream &operator<<(ostream &os, componenttype &ct)
{
  return os << "Component type : " << ct.name << "\n";
}

ostream &operator<<(ostream &os, project &p)
{
  return os << "Project\nSymbols :\n" << *p.symbols << "\nComponent Types :\n" << *p.componenttypes << "\nMain Component :" << p.maincomponent;
}


