/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __PROJECT_H__
#define __PROJECT_H__

#ifdef __linux__
//#define __DEBUG 1
#endif

#ifdef __DEBUG
#define DEBUG(x) { x }
#else
#define DEBUG(x) 
#endif

#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#ifdef __linux__
#include <ctype.h>
#endif
#include "list.h"
#include "symboltable.h"
#include "number.h"
#include "error.h"

#ifdef __linux__
#define random(x) (random() % (x))
#endif

#define MAXSYMBOLNAMELENGTH 200
//#define TIMESTEP 1
#define MAXITER 100

#define RUN_MODEMASK 48
#define RUN_GO 16
#define RUN_CONTANIM 32
#define RUN_ANIM 48
#define RUN_STOP 0

#define RUN_UNITMASK 12
#define RUN_TICK 4
#define RUN_STEP 8
#define RUN_TRACE 12

#define RUN_ANIMMASK 3
#define RUN_NORMANIM 0
#define RUN_FASTANIM 1
#define RUN_SLOWANIM 2
#define RUN_ONCE 3

#define MAXIDLEFUNCTIONLOOPS 50

class macrocomponenttype;
class symbol;
class symbolconstructor;
class macrocomponent;
class component;
class net;
class pin;
class attribute;
class plug;
class variable;
class displaybox;
class disassembly;
class realsymbol;
class Oscillo;
class project;
class array;
template <class t> class genericcomponenttype;

typedef list<pin> pinlist;
typedef list<net> netlist;

#ifdef __linux__
  #include "widget.h"
#else
  #include "snprintf.h"
  #include "widget_win.h"
#endif
#include "parser.h"
#include "symbol.h"
#include "attribute.h"
#include "componenttype.h"
#include "displaybox.h"
#include "component.h"
#include "pin.h"
#include "bus.h"
#include "variable.h"
#include "disassembly.h"
#include "break.h"
#include "real.h"
#include "array.h"
#include "Scenix.h"
#include "HexFile.h"
#include "lstfile.h"
#include "sxopcode.h"

#include "basecomponents.h"
#include "Oscillo.h"

void RegisterComponentType(componenttype *ct, char *name);
extern pin *N;
extern window *w;

class project : public symbol
{
  unsigned int BreakFlag : 1;
  unsigned int StepFlag : 1;
  unsigned int TraceFlag : 1;
  int StepID;

  real time;
  real chrono,lasttime; //peut tre remis  zro
  //  real TimeStep;
  symbol *_getlocalsymbol(symbolname *s);
  realptr timesymbol;
  realptr chronosymbol;
  int MaxIter;
  //  outpin VCC, GND;

public:
  int AnimateCount;
  int FastAnimateSpeed;
  int SlowAnimateDelay;
  int RunMode;

  void Break() { BreakFlag = 1; }
  void Step() { StepFlag = 1; }
  void Trace() { TraceFlag = 1; }
  int GetStepID() { return StepID; }

  symbolwidget *getwidget() { return new projectwidget(getattributes(), this); }
  project();
  real GetTime() { return time; };
  componenttype *GetComponentType(int name);
  plugtype *GetPlugType(int name);
  int IdleLoop();
  int Tick(int maxstep);
  void StartInterface();
  void UnlinkWindow(window *w);
  void RedrawAll();
  void Reset();

  list<symbol> *symbols;
  list<componenttype> *componenttypes;
  component *maincomponent;
  list<symbolconstructor> *windowrootconstructors;
  list<window> *windows;
  list<plugtype> *plugtypes;

  int DispOptDebugInfo; // ####
};

extern project Project;
extern real TimeStep;
extern char *ProgramVersion;

ostream &operator<<(ostream &os, symbol &s);
ostream &operator<<(ostream &os, componenttype &);
template <class t>
ostream &operator<<(ostream &os, list<t> &_l);
ostream &operator<<(ostream &os, project &p);



#endif
